<?php
/* vim: set noexpandtab tabstop=2 softtabstop=2 shiftwidth=2: */

/**
 * MXInfoFetcher - Fetch info/records for TM2/SM/QM maps from ManiaExchange
 * Created by Xymph <tm@gamers.org> based on https://api2.mania-exchange.com/
 * Derived from TMXInfoFetcher
 *
 * v2.0: Updated to use the current ManiaExchange API; added $gbxname, $gbxlogin,
 *       $authtime, $vehicle, $ismp4, $ghostblks, $embedcnt, $embedsize,
 *       $sizewarn; removed deprecated $lbrating, $replaytyp, $ratingex;
 *       added 'vehicle' & 'exebld' fields to $recordlist entries
 * v1.9: Added https support to get_file
 * v1.8: Fixed constructor for PHP 8.x, updated URLs to https
 * v1.7: Added $titlepack (TM2/SM)
 * v1.6: Allowed 24-char UIDs too
 * v1.5: Added $maptype (TM2/SM)
 * v1.4: Updated to use MX API v2.0 and add/fix support for SM; added
 *       $trkvalue (TM2, equals deprecated $lbrating), $unlimiter (TM2/SM),
 *       $rating/$ratingex/$ratingcnt (SM)
 * v1.3: Added URLs to downloadable replays
 * v1.2: Added the replays list in $recordlist
 * v1.1: Allowed 25-char UIDs too
 * v1.0: Initial release
 */
class MXInfoFetcher {

	public $section, $prefix, $uid, $id, $records, $error,
		$name, $gbxname, $userid, $author, $gbxlogin, $authtime,
		$uploaded, $updated, $type, $maptype, $titlepack, $style, $envir, $mood,
		$dispcost, $lightmap, $vehicle, $modname, $exever, $exebld, $ismp4,
		$routes, $length, $unlimiter, $laps, $diffic, $trkvalue,
		$replayid, $replaycnt, $acomment, $awards, $comments, $ghostblks,
		$embedcnt, $embedsize, $sizewarn, $rating, $ratingcnt,
		$pageurl, $replayurl, $imageurl, $thumburl, $dloadurl, $recordlist;

	/**
	 * Fetches all available data for a ManiaExchange map
	 *
	 * @param String $game
	 *        MX section for 'TM2', 'SM', 'QM'
	 * @param String $id
	 *        The map UID to search for (if a 24-27 char alphanum string),
	 *        otherwise the MX ID to search for (if a number)
	 * @param Boolean $records
	 *        If true, the script also returns the world records (max. 10)
	 * @return MXInfoFetcher
	 *        If $error is not an empty string, it's an error message
	 */
	public function __construct($game, $id, $records) {

		$this->section = $game;
		switch ($game) {
		case 'TM2':
			$this->prefix = 'tm';
			break;
		case 'SM':
			$this->prefix = 'sm';
			break;
		case 'QM':
			$this->prefix = 'qm';
			break;
		default:
			$this->prefix = '';
			return;
		}

		$this->error = '';
		$this->records = $records;
		// check for UID string
		if (preg_match('/^\w{24,27}$/', $id)) {
			$this->uid = $id;
			$this->getData(true);
		// check for MX ID
		} elseif (is_numeric($id) && $id > 0) {
			$this->id = intval($id);
			$this->getData(false);
		}
	}  // __construct

	public static function __set_state($import) {

		$mx = new MXInfoFetcher('', 0, false);

		$mx->section   = $import['section'];
		$mx->prefix    = $import['prefix'];
		$mx->uid       = $import['uid'];
		$mx->id        = (int)$import['id'];
		$mx->records   = (bool)$import['records'];
		$mx->error     = '';
		$mx->name      = $import['name'];
		$mx->gbxname   = isset($import['gbxname']) ? $import['gbxname'] : $import['name'];
		$mx->userid    = (int)$import['userid'];
		$mx->author    = $import['author'];
		$mx->gbxlogin  = isset($import['gbxlogin']) ? $import['gbxlogin'] : $import['author'];
		$mx->authtime  = isset($import['authtime']) ? $import['authtime'] : 0;
		$mx->uploaded  = $import['uploaded'];
		$mx->updated   = $import['updated'];
		$mx->type      = $import['type'];
		$mx->maptype   = isset($import['maptype']) ? $import['maptype'] : '';
		$mx->titlepack = isset($import['titlepack']) ? $import['titlepack'] : '';
		$mx->style     = $import['style'];
		$mx->envir     = $import['envir'];
		$mx->mood      = $import['mood'];
		$mx->dispcost  = (int)$import['dispcost'];
		$mx->lightmap  = (int)$import['lightmap'];
		$mx->vehicle   = isset($import['vehicle']) ? $import['vehicle'] : '';
		$mx->modname   = $import['modname'];
		$mx->exever    = $import['exever'];
		$mx->exebld    = $import['exebld'];
		$mx->ismp4     = isset($import['ismp4']) ? (bool)$import['ismp4'] : false;
		$mx->routes    = (string)$import['routes'];
		$mx->length    = (string)$import['length'];
		$mx->unlimiter = isset($import['unlimiter']) ? (bool)$import['unlimiter'] : false;
		$mx->laps      = (int)$import['laps'];
		$mx->diffic    = (string)$import['diffic'];
		$mx->trkvalue  = isset($import['trkvalue']) ? (int)$import['trkvalue'] : 0;
		$mx->replayid  = (int)$import['replayid'];
		$mx->replaycnt = (int)$import['replaycnt'];
		$mx->acomment  = $import['acomment'];
		$mx->awards    = (int)$import['awards'];
		$mx->comments  = (int)$import['comments'];
		$mx->ghostblks = isset($import['ghostblks']) ? (bool)$import['ghostblks'] : false;
		$mx->embedcnt  = isset($import['embedcnt']) ? (int)$import['embedcnt'] : 0;
		$mx->embedsize = isset($import['embedsize']) ? (int)$import['embedsize'] : 0;
		$mx->sizewarn  = isset($import['sizewarn']) ? (bool)$import['sizewarn'] : false;
		$mx->rating    = isset($import['rating']) ? (float)$import['rating'] : 0.0;
		$mx->ratingcnt = isset($import['ratingcnt']) ? (int)$import['ratingcnt'] : 0;
		$mx->pageurl   = $import['pageurl'];
		$mx->replayurl = $import['replayurl'];
		$mx->imageurl  = $import['imageurl'];
		$mx->thumburl  = $import['thumburl'];
		$mx->dloadurl  = $import['dloadurl'];
		$mx->recordlist = null;

		return $mx;
	}  // __set_state

	private function getData($isuid) {

		// get map info
		$url = "https://{$this->prefix}.mania-exchange.com/api/maps/get_map_info/";
		if ($isuid)
			$url .= "uid/{$this->uid}";
		else
			$url .= "id/{$this->id}";
		$file = $this->get_file($url);
		if ($file === false) {
			$this->error = 'Connection or response error on ' . $url;
			return;
		} elseif ($file === -1) {
			$this->error = 'Timed out while reading data from ' . $url;
			return;
		} elseif ($file == '') {
			$this->error = 'No data returned from ' . $url;
			return;
		}

		// process map info
		$mx = json_decode($file);
		if ($mx === null) {
			$this->error = 'Cannot decode JSON data from ' . $url;
			return;
		}
		if (empty($mx)) {
			$this->error = 'No data returned from ' . $url;
			return;
		}

		if ($isuid)
			$this->id      = $mx->TrackID;
		else
			$this->uid     = $mx->TrackUID;
		$this->name      = $mx->Name;
		$this->gbxname   = $mx->GbxMapName;
		$this->userid    = $mx->UserID;
		$this->author    = $mx->Username;
		$this->gbxlogin  = $mx->AuthorLogin;
		$this->authtime  = $mx->AuthorTime;
		$this->uploaded  = $mx->UploadedAt;
		$this->updated   = $mx->UpdatedAt;
		$this->type      = $mx->TypeName;
		$this->maptype   = $mx->MapType;
		$this->titlepack = $mx->TitlePack;
		$this->style     = $mx->StyleName;
		$this->envir     = $mx->EnvironmentName;
		$this->mood      = $mx->Mood;
		$this->dispcost  = $mx->DisplayCost;
		$this->lightmap  = $mx->Lightmap;
		$this->vehicle   = $mx->VehicleName;
		$this->modname   = $mx->ModName;
		$this->exever    = $mx->ExeVersion;
		$this->exebld    = $mx->ExeBuild;
		$this->ismp4     = $mx->IsMP4;
		$this->routes    = (string)$mx->RouteName;
		$this->length    = (string)$mx->LengthName;
		$this->unlimiter = $mx->UnlimiterRequired;
		$this->laps      = $mx->Laps;
		$this->diffic    = (string)$mx->DifficultyName;
		$this->trkvalue  = $mx->TrackValue;
		$this->replayid  = (int)$mx->ReplayWRID;
		$this->replaycnt = $mx->ReplayCount;
		$this->acomment  = $mx->Comments;
		$this->awards    = $mx->AwardCount;
		$this->comments  = $mx->CommentCount;
		$this->ghostblks = $mx->HasGhostBlocks;
		$this->embedcnt  = $mx->EmbeddedObjectsCount;
		$this->embedsize = $mx->EmbeddedItemsSize;
		$this->sizewarn  = $mx->SizeWarning;
		$this->rating    = $mx->RatingVoteAverage;
		$this->ratingcnt = $mx->RatingVoteCount;

		$search = array(chr(31), '[b]', '[/b]', '[i]', '[/i]', '[u]', '[/u]', '[url]', '[/url]');
		$replace = array('<br/>', '<b>', '</b>', '<i>', '</i>', '<u>', '</u>', '<i>', '</i>');
		$this->acomment  = str_ireplace($search, $replace, $this->acomment);
		$this->acomment  = preg_replace('/\[url=.*\]/', '<i>', $this->acomment);

		$this->pageurl   = "https://{$this->prefix}.mania-exchange.com/maps/view/{$this->id}";
		$this->imageurl  = $mx->HasScreenshot ? "https://{$this->prefix}.mania-exchange.com/maps/image/{$this->id}" : '';
		$this->thumburl  = $mx->HasThumbnail ? "https://{$this->prefix}.mania-exchange.com/maps/thumbnail/{$this->id}" : '';
		$this->dloadurl  = "https://{$this->prefix}.mania-exchange.com/maps/download/{$this->id}";

		if ($this->replayid > 0)
			$this->replayurl = "https://{$this->prefix}.mania-exchange.com/replays/download/{$this->replayid}";
		else
			$this->replayurl = '';

		// fetch records too?
		$this->recordlist = array();
		if ($this->prefix == 'tm' && $this->records) {
			// get replays info (max. 10 entries)
			$limit = 10;
			$url = "https://{$this->prefix}.mania-exchange.com/api/replays/get_replays/{$this->id}/{$limit}";
			$file = $this->get_file($url);
			if ($file === false || $file === -1 || $file == '')
				// no error message as main info was already fetched
				return;

			// process replays info
			$mx = json_decode($file);
			if ($mx === null) {
				$this->error = 'Cannot decode JSON data from ' . $url;
				return;
			}

			foreach ($mx as $replay) {
				$this->recordlist[] = array(
					'replayid'   => $replay->ReplayID,
					'userid'     => $replay->UserID,
					'username'   => $replay->Username,
					'uploadedat' => $replay->UploadedAt,
					'replaytime' => $replay->ReplayTime,
					'stuntscore' => $replay->StuntScore,
					'respawns'   => $replay->Respawns,
					'beaten'     => $replay->Beaten,
					'percentage' => $replay->Percentage,
					'replaypnts' => $replay->ReplayPoints,
					'nadeopnts'  => $replay->NadeoPoints,
					'vehicle'    => $replay->PlayerModel,
					'exebld'     => $replay->ExeBuild,
					'replayurl'  => "https://{$this->prefix}.mania-exchange.com/replays/download/{$replay->ReplayID}",
				);
			}
		}
	}  // getData

	// Simple HTTP(S) Get function with timeout
	// ok: return string || error: return false || timeout: return -1
	private function get_file($url) {

		$url = parse_url($url);
		$port = isset($url['port']) ? $url['port'] :
		        ($url['scheme'] == 'https' ? 443 : 80);
		$query = isset($url['query']) ? '?' . $url['query'] : '';

		$fp = @fsockopen(($url['scheme'] == 'https' ? 'ssl://' : '') . $url['host'],
		                 $port, $errno, $errstr, 4);
		if (!$fp)
			return false;

		fwrite($fp, 'GET ' . $url['path'] . $query . " HTTP/1.0\r\n" .
		            'Host: ' . $url['host'] . "\r\n" .
		            'Content-Type: application/json' . "\r\n" .
		            'User-Agent: MXInfoFetcher (' . PHP_OS . ")\r\n\r\n");
		stream_set_timeout($fp, 4);
		$res = '';
		$info = array();
		$info['timed_out'] = false;
		while (!feof($fp) && !$info['timed_out']) {
			$res .= fread($fp, 512);
			$info = stream_get_meta_data($fp);
		}
		fclose($fp);

		if ($info['timed_out']) {
			return -1;
		} else {
			if (substr($res, 9, 3) != '200')
				return false;
			$page = explode("\r\n\r\n", $res, 2);
			return trim($page[1]);
		}
	}  // get_file
}  // class MXInfoFetcher

